Package: snpStats
Title: SnpMatrix and XSnpMatrix classes and methods
Version: 1.56.0
Date: 2022-07-27
Author: David Clayton <dc208@cam.ac.uk> 
Description: Classes and statistical methods for large SNP association studies. This extends the earlier snpMatrix package, allowing for uncertainty in genotypes.
Maintainer: David Clayton <dc208@cam.ac.uk>
Depends: R(>= 2.10.0), survival, Matrix, methods
Imports: graphics, grDevices, stats, utils, BiocGenerics, zlibbioc
Suggests: hexbin
License: GPL-3
Collate: ss.R contingency.table.R convert.R compare.R glm-test.R
        imputation.R indata.R long.R misc.R ld.R mvtests.R pedfile.R
        outdata.R plink.R qc.R qq-chisq.R single.R tdt-single.R
        structure.R xstuff.R
LazyLoad: yes
biocViews: Microarray, SNP, GeneticVariability
Packaged: 2024-10-30 02:57:03 UTC; biocbuild
git_url: https://git.bioconductor.org/packages/snpStats
git_branch: RELEASE_3_20
git_last_commit: 92d244d
git_last_commit_date: 2024-10-29
Repository: Bioconductor 3.20
Date/Publication: 2024-10-29
NeedsCompilation: yes
