useDynLib("snpStats")
import(methods)
import(zlibbioc)
importFrom(graphics, abline, axis, barplot, legend, lines, text, mtext, par,
           plot, points, polygon)
importFrom(grDevices, heat.colors)
importFrom(stats, model.matrix, model.offset, model.response, contrasts,
           model.weights, pchisq, qbeta, qchisq, terms)
importFrom(survival, cluster, strata, untangle.specials)
importFrom(utils, download.file, flush.console, head, read.table, write.table)
importFrom(BiocGenerics, cbind)
importFrom(BiocGenerics, rbind)
importClassesFrom(Matrix, dsCMatrix, dgCMatrix)

exportClasses(SnpMatrix, XSnpMatrix, ImputationRules,
              SingleSnpTests, SingleSnpTestsScore,
              GlmTests, GlmEstimates)
exportMethods(cbind, chi.squared, deg.freedom, effect.sign, is.na, 
              names, pool2, p.value, rbind, show, summary, switch.alleles,
              plot, sample.size, effective.sample.size,
              "[", "[<-", coerce, initialize)
export(can.impute, convert.snpMatrix, convert.snpMatrix.dir,
       filter.rules, Fst, glm.test.control, g2post, ibsCount, ibsDist,
       impute.snps, imputation.maf, imputation.r2, imputation.nsnp, ld, mean2g,
       misinherits, mvtests, plotUncertainty, pool, post2g, pp, qq.chisq, 
       read.beagle, read.impute, read.long, read.mach, read.pedfile,
       read.plink, read.snps.long, random.snps,
       row.summary, col.summary,
       single.snp.tests, sm.compare,
       snp.cor, snp.imputation, snp.lhs.tests, snp.lhs.estimates,
       snp.post.multiply, snp.pre.multiply,
       snp.rhs.tests, snp.rhs.estimates, tdt.snp,
       test.allele.switch, write.plink, write.SnpMatrix, xxt)
